"""Tests for pip-help package."""

import os
from subprocess import getstatusoutput

prg = "src/pip_help/main.py"
target_path = (os.path.join(os.path.expanduser("~"),  "pip_help_logs"))


def test_pip_help_exists():
    """Checks if the pip-help main file exists."""

    assert os.path.isfile(prg)


def test_usage():
    """Checks if the usage message is displayed."""

    for flag in ["-h", "--help"]:
        rv, out = getstatusoutput(f"python {prg} {flag}")
        assert rv == 0
        assert out.lower().startswith("usage")

def test_no_args():
    """Checks if the warning message is displayed when no arguments are provided."""

    rv, out = getstatusoutput(f"python {prg}")
    assert rv == 0
    assert out.lower().startswith("warning")


def test_dir_creation():
    """
    Checks if the directory is created.
    """

    rv, out = getstatusoutput(f"python {prg} --install requests")
    assert rv == 0
    assert os.path.isdir(target_path)


def test_dir_deletion():
    """
    Checks if the directory is deleted after uninstalling the last package installed by pip-help.
    """

    rv, out = getstatusoutput(f"python {prg} --remove requests")
    assert rv == 0
    assert not os.path.isdir(target_path)

    



import argparse
import os
import shutil


from pip_help.pip_func import (
    install,
    installed_packages_list,
    uninstall,
    delete_pip_cache,
    log_dest_path,
    pkg_on_pypi,
)


def get_args():
    parser = argparse.ArgumentParser(
        prog="pip help",
        description="Provide support for installing packages and dependendable packages temporarily.",
        epilog="If you followed the steps above, you have succesfully made your system cleaner and more efficient.",
    )
    group = parser.add_mutually_exclusive_group()

    group.add_argument(
        "--install",
        "-i",
        type=str,
        help="The name of the package you want to install.",
        nargs="+",
    )
    group.add_argument(
        "--remove",
        "-r",
        type=str,
        help="The name of the package you want to remove.",
        nargs="+",
    )
    return parser.parse_args()


def main():
    args = get_args()
    package_install = args.install
    package_remove = args.remove

    if package_install:
        for package in package_install:
            pkg_exist = pkg_on_pypi(package)
            if pkg_exist is True:
                try:
                    print("*" * 66)
                    print(f"Installing {package}...")
                    install(package)
                except TypeError as e:
                    print(f"Error: {e}")
                finally:
                    installed_packages_list(package)
            else:
                print("*" * 66)
                print(
                    f"WARNING: Make sure {package} is a valid package name. Please try again."
                )
            print(
                f"NOTE!: {package} is not a valid package name."
                if not pkg_exist
                else ""
            )
    else:
        if package_remove:
            for package in package_remove:
                try:
                    print("*" * 45)
                    uninstall(package)
                    print("*" * 45)
                    cache = input(
                        "Do you want to delete the pip cache? (y/n): "
                    ).lower()
                    while cache not in ["y", "n"]:
                        cache = input(
                            "Please enter a valid input. Do you want to delete the pip cache? (y/n): "
                        ).lower()
                    if cache == "y":
                        delete_pip_cache()
                        os.remove((log_dest_path() + f"/pip_{package}_list.txt"))
                    else:
                        pass

                except TypeError as e:
                    print(f"Error: {e}")
        else:
            print(
                'WARNING: No argument provided. Please provide an argument. Use "--help/-h" for more information.'
            )

        if os.path.exists(log_dest_path()):
            if len(os.listdir(log_dest_path())) == 1:
                shutil.rmtree(log_dest_path())
            else:
                pass


if __name__ == "__main__":
    main()



import subprocess
import os
from pathlib import Path
from typing import TypeVar
import requests


def log_dest_path() -> Path:
    log_dest_path = os.path.join(os.path.expanduser("~"), "pip_help_logs")
    if not os.path.exists(log_dest_path):
        os.makedirs(log_dest_path)
    with open(f"{log_dest_path}/readme.md", "w") as file:
        file.write(
            "This directory contains logs of all the packages installed and removed using the pip-help command. Remove this directory if you are not using the pip-help command anymore."
        )
    return log_dest_path


File = TypeVar("File", str, Path)


def install(pip_package: str) -> None:
    log_path = os.path.join(log_dest_path(), f"pip_{pip_package}.txt")
    package = subprocess.call(
        ["pip", "install", pip_package, "--log", log_path, "--quiet"]
    )
    print(f"Saving logs temporarily at: {log_dest_path()}")
    return package


def installed_packages_list(pip_package: str) -> File:
    target_path = os.path.join(log_dest_path(), f"pip_{pip_package}.txt")
    validation = False
    with open(target_path, "r") as file:
        possible_packages = []
        dependent_packages = []

        for line in file.readlines():
            target_line = line.lower().split(" ")
            word = "installing"
            if word in target_line:
                possible_packages.append(target_line)
        file.close()

        if len(possible_packages) == 0:
            print("-" * 66)
            print(
                f'WARINING: Package {pip_package.upper()} is already installed. it is recommended to uninstall "{pip_package}" and related packages before installing it again. To uninstall "{pip_package}" and related packages, run the command: \'pip-help --remove/-r {pip_package}\''
            )
            os.remove(target_path)

        else:
            for i, k in enumerate(possible_packages[0]):
                if k == "packages:":
                    target_packages = possible_packages[0][i + 1 :]
                    for target in target_packages:
                        package_name = target.strip().split(",")[0]
                        dependent_packages.append(package_name)
            print(f"{dependent_packages[-1]} is successfully installed!")

            with open((log_dest_path() + f"/pip_{pip_package}_list.txt"), "w") as file:
                try:
                    for pkg in dependent_packages:
                        file.write(f"{pkg}\n")
                        validation = True
                except Exception as e:
                    print(f"Error: {e}")

                if validation:
                    try:
                        os.remove(target_path)
                    except PermissionError as e:
                        print(
                            f"Error: {e}. Please close the file and try again to remove the file."
                        )


def uninstall(pip_package: str) -> None:
    log_path = log_dest_path() + f"/pip_{pip_package}_list.txt"
    if not os.path.exists(log_path):
        raise TypeError(
            f"The package {pip_package.upper()}, you are trying to remove is not installed."
        )
    else:
        package_uninstalled = subprocess.call(
            ["pip", "uninstall", "-r", log_path, "--yes"]
        )
        return package_uninstalled


def delete_pip_cache() -> None:
    clear_cache = subprocess.call(["pip", "cache", "purge"])
    return clear_cache


def pkg_on_pypi(pip_package: str) -> bool:
    request_api = requests.get(f"https://pypi.org/pypi/{pip_package}/json/")
    response = request_api.status_code
    if response == 200:
        return True
    else:
        return False
